--------------------------------------------------------------------------
-- Skrypt strojenia bazy danych CDN
-- Wszystkie operacje wykonywane s na wskazanej przez kursor bazie danych
--------------------------------------------------------------------------

SET ARITHABORT ON 
SET QUOTED_IDENTIFIER ON 

-- Kontrola wersji silnika SQL
DECLARE @Version varchar(500)
SET @Version = (SELECT @@VERSION)

-- Kursor wyszukujcy bazy CDN
DECLARE @Nazwa_bazy VARCHAR(40)

IF @Version LIKE '%2000%'
	BEGIN
		DECLARE KursorBaz CURSOR FORWARD_ONLY FOR 
		SELECT name
		FROM sysdatabases
		WHERE name LIKE 'CDN%'
	END
	ELSE BEGIN
		DECLARE KursorBaz CURSOR FORWARD_ONLY FOR 
		SELECT name
		FROM sys.databases
		WHERE name LIKE 'CDN%'
	END

OPEN KursorBaz
	WHILE 1 = 1
		BEGIN
			FETCH NEXT FROM KursorBaz INTO  @Nazwa_bazy
			IF @@FETCH_STATUS <> 0 BREAK

-- Dla kadej bazy wykonywane jest strojenie

PRINT '/------------------------------------------\'
PRINT '|    Strojenie bazy ' +@Nazwa_Bazy
PRINT '\------------------------------------------/' + CHAR(13)

-- Deklaracja zmiennej cmd dla wykonywania polece
DECLARE @CMD NVARCHAR(500)

-------------------------------------------------------
-- Krok 1. Kontrola poprawnoci bazy
-------------------------------------------------------

DECLARE @Error int
DECLARE @ErrorDesc nvarchar(2048)

PRINT '/------------------------------------------\'
PRINT '|    Kontrola DBCC CHECKDB bazy danych     |'
PRINT '\------------------------------------------/' + CHAR(13)

DBCC CHECKDB (@Nazwa_Bazy) WITH NO_INFOMSGS
SET @Error = @@ERROR

IF @Error > 0 -- Po wystpieniu bdu wywietlane jest ostrzeenie, dalsze operacje nie s wykonywane
	BEGIN
		SET @ErrorDesc = (SELECT description FROM master..sysmessages WHERE error = @Error AND msglangid = 1033)
		PRINT 'Kontrola DBCC CHECKDB wykazaa bd, dalsze operacje zostay przerwane. ' + CHAR(13) + 'Skontroluj poprawno bazy danych wykonujc polecenie DBCC CHECKDB bez parametru WITH NO_INFOMSGS aby uzyska dokadne wyniki.'
		PRINT 'Charakter bdu: " ' + @ErrorDesc + ' "'
		CLOSE      KursorBaz
		DEALLOCATE KursorBaz
	END
	
	ELSE BEGIN -- Wykonywane s kolejne operacje
		PRINT CHAR(13) + 'Kontrola DBCC CHECKDB nie wykazaa bdw. ' + CONVERT (varchar(20) , GETDATE(), 120 )
		
-------------------------------------------------------
-- Krok 2. Odbudowa indeksw bazy
-------------------------------------------------------

PRINT CHAR(13)
PRINT '/------------------------------\'
PRINT '|    Odbudowa indeksw bazy    |'
PRINT '\------------------------------/' + CHAR(13)

-- Deklaracja zmiennych potrzebnych do wykonania operacji
DECLARE @Tabela VARCHAR(255)  
DECLARE @FillFactor INT 

-- Ustalenie wartoci zmiennych
SET @FillFactor = 80 -- ustalamy waro Fill Factor

SET @CMD = '
DECLARE KursorTabelStatystykiIndeksy CURSOR FOR 
SELECT TABLE_CATALOG + ''.'' + TABLE_SCHEMA + ''.'' + TABLE_NAME as TableName   
FROM ' +@Nazwa_bazy +'.INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE = ''BASE TABLE''
'

EXEC (@CMD)

OPEN KursorTabelStatystykiIndeksy 
FETCH NEXT FROM KursorTabelStatystykiIndeksy INTO @Tabela   
WHILE @@FETCH_STATUS = 0   

	BEGIN
		IF @Version LIKE '%2000%'
			BEGIN 
				DBCC DBREINDEX(@Tabela,' ',@fillfactor) WITH NO_INFOMSGS
			END   
		ELSE
		    BEGIN     
				 SET @CMD = 'ALTER INDEX ALL ON ' + @Tabela + ' REBUILD WITH (FILLFACTOR = ' + CONVERT(VARCHAR(3),@FillFactor) + ')'  
				 EXEC (@CMD)  
			END
	FETCH NEXT FROM KursorTabelStatystykiIndeksy INTO @Tabela  
	END   

CLOSE KursorTabelStatystykiIndeksy   
DEALLOCATE KursorTabelStatystykiIndeksy

PRINT CHAR(13) + 'Odbudowa indeksw bazy zakoczona. ' + CONVERT (varchar(20) , GETDATE(), 120 )

-------------------------------------------------------
-- Krok 3. Odbudowa statystyk bazy
-------------------------------------------------------

PRINT CHAR(13)
PRINT '/-------------------------------\'
PRINT '|   Odbudowa statystyk bazy     |'
PRINT '\-------------------------------/' + CHAR(13)

SET @CMD = '
DECLARE KursorTabelStatystyki CURSOR FOR 
SELECT TABLE_CATALOG + ''.'' + TABLE_SCHEMA + ''.'' + TABLE_NAME as TableName   
FROM ' +@Nazwa_bazy +'.INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE = ''BASE TABLE''
'

EXEC (@CMD)

OPEN KursorTabelStatystyki 
FETCH NEXT FROM KursorTabelStatystyki INTO @Tabela   
WHILE @@FETCH_STATUS = 0   

	BEGIN
		SET @CMD = 'UPDATE STATISTICS ' + @Tabela + ' WITH FULLSCAN '  
		EXEC (@CMD)  
	FETCH NEXT FROM KursorTabelStatystyki INTO @Tabela  
	END   

CLOSE KursorTabelStatystyki   
DEALLOCATE KursorTabelStatystyki

PRINT CHAR(13) + 'Odbudowa statystyk bazy zakoczona. ' + CONVERT (varchar(20) , GETDATE(), 120 )

-------------------------------------------------------
-- Krok 4. Kopia bazy danych
-------------------------------------------------------

PRINT CHAR(13)
PRINT '/----------------------------------------\'
PRINT '|   Wykonywanie kopii bezpieczestwa     |'
PRINT '\----------------------------------------/' + CHAR(13)

-- Deklaracja zmiennych dla kopii bezpieczestwa
DECLARE @Sciezka    VARCHAR(260)
DECLARE @PlikArchiw VARCHAR(300)
DECLARE @NazwaBazyArchiwum VARCHAR(200)

-- Rozrnienie nazw dla bazy konfiguracyjnej oraz firmowej
IF @Nazwa_bazy LIKE 'CDN_KNF%' OR @Nazwa_bazy LIKE 'CDN_Konfiguracja%'
	BEGIN
		SET @NazwaBazyArchiwum = 'OPT!MA KONFIGURACJA'
	END
	ELSE
	BEGIN
		SET @NazwaBazyArchiwum = 'OPT!MA BAZA DANYCH'
	END

-- cieka dostpu do miejsca, w ktrym kopie bezpieczestwa bd zapisywane
SET @Sciezka = N'C:\BACKUP\' -- Tu wpisz ciek dostpu do katalogu dla kopii baz

-- Wykonanie skryptu
        SET @PlikArchiw = @Sciezka + @Nazwa_bazy + '_' + 
         REPLACE(REPLACE(REPLACE(CONVERT(CHAR(16), GETDATE(), 120), ' ', ''), '-', ''), ':', '') + '.BAC'
        BACKUP DATABASE @Nazwa_bazy TO DISK = @PlikArchiw WITH INIT, NAME = @NazwaBazyArchiwum, SKIP, NOFORMAT, DESCRIPTION = ''
        IF @@error>0 
          BEGIN 
            PRINT ''
            PRINT 'Kopia bezpieczestwa bazy ' + @Nazwa_Bazy + ' nie wykonana, wystpiy problemy ! ' + CONVERT (varchar(20) , GETDATE(), 120 )
            PRINT ''
          END
        ELSE
          BEGIN 
            RESTORE VERIFYONLY FROM DISK = @PlikArchiw WITH FILE = 1
            IF @@error>0
              BEGIN
                PRINT ''
                PRINT 'Kopia bezpieczestwa bazy ' + @Nazwa_Bazy + ' nie wykonana, wystpiy problemy ! ' + CONVERT (varchar(20) , GETDATE(), 120 )
                PRINT ''
              END
            ELSE
              BEGIN
                PRINT ''
                PRINT 'Kopia bezpieczestwa bazy ' + @Nazwa_bazy + ' (' + @PlikArchiw + ')' + ' wykonana prawidowo ! ' + CONVERT (varchar(20) , GETDATE(), 120 )
                PRINT ''
              END
          END

END
END
CLOSE      KursorBaz
DEALLOCATE KursorBaz